// Simple association tester and example usage code
#include "assoc.h"
#include <stdio.h>


// two dummy classes that might have an association to each other.
class Edge;

class Vertex
{
public:
  float x, y, z ;
  assoc<Vertex,Edge> edge;

  Vertex() : edge(this) {
    x = 0.0f;
	y = 0.0f;
	z = 0.0f;
  }
};

class Edge
{
public:
  float angle;

  assoc<Edge,Vertex> vertex;

  Edge() : vertex(this) {
    angle = 0;
  }
};


// Test/example usage 'main' for simple associations.
int main () 
{
  Vertex v1;
  Edge e1;
  Vertex *pv;
  Edge *pe;

  // Set up an edge and vertices 
  v1.x = 24.0f;
  v1.y = 14.0f;
  v1.z =  4.0f;
  e1.angle = 0.5;

  // Associate v1 and e1
  v1.edge.link(&e1.vertex);

  // Test association v1/e1 directly
  printf("This should be %g => %g\n", v1.x, e1.vertex.get()->x);
  printf("and this should be %g => %g\n", e1.angle, v1.edge.get()->angle);

  // That interface can be clumbsy so it can be cleaner to use the
  // overloaded cast operation.
  float i = ((Edge *)v1.edge)->angle;

  // Or for the cleanest referencing, use pointers
  pv = e1.vertex;
  pe = v1.edge;

  // Print out verification of expected results
  printf("This should be %g => %g\n", v1.y, pv->y);
  printf("and this should be %g => %g\n", e1.angle, pe->angle);

  // Now test clearing the association
  pe->vertex.unlink(&pv->edge);

  if (e1.vertex == NULL && v1.edge == NULL)
    printf("We successfully cleared the association\n");
  else
    printf("Test FAILED.\n");

  // Re-create the association.
  v1.edge.link(&e1.vertex);

  // Now break it another way
  v1.edge.unlink();

  // Test again
  if (e1.vertex == NULL && v1.edge == NULL)
    printf("We successfully cleared the association\n");
  else
    printf("Test FAILED.\n");

  // Now test the destructor
  Vertex *v2 = new Vertex;

  v2->z = 3.1415f;
  v2->edge.link(&e1.vertex);
  pv = e1.vertex;

  // Print out verification of expected results
  printf("This should be %g => %g\n", v2->z, pv->z);
  printf("and this should be %g => %g\n", e1.angle, pe->angle);

  delete v2;

  if (e1.vertex == NULL)
    printf("We successfully cleared the association\n");
  else
    printf("Test FAILED.\n");

  return 0;
}
